#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from htf.daqmx.core import DAQError
from htf.daqmx.task import Task
from ctypes import *  # noqa: F403
from htf.daqmx.api import (
    uInt32,
    float64,
    DAQmx_Val_CountUp,
    DAQmx_Val_CountDown,
    DAQmx_Val_Rising,
    DAQmx_Val_Falling,
)
from htf.mixins.delegation import DelegateMixin


class EdgeCounterInput(DelegateMixin):
    """
    ``EdgeCounterInput`` is used to access a counter input on a DAQmx device.
    The counter is 32 bit wide.

    A DAQmx task is automatically created and deleted in the constructor and destructor.
    """

    def __init__(
        self, descriptor: str, initial_count: int = 0, rising_edge: bool = True, count_up: bool = True
    ) -> None:
        """
        Args:
            descriptor (str): The DAQmx device and line descriptor, eg.
                `cDaq1Mod1/ctr0`.
            initial_count=0 (int): The initial count.
            rising_edge=True (bool): If set to True rising edges are counted
                else falling edges are counted.
            count_up=True (bool): If set to True the counter direction is up
                else it is down.
        """
        task_name = "CTR_" + descriptor.replace("/", "_")
        self.count = initial_count
        self.offset = 0
        self.data = uInt32(0)
        self.task = Task(task_name)

        self.edge = DAQmx_Val_Rising
        if not rising_edge:
            self.edge = DAQmx_Val_Falling

        self.count_direction = DAQmx_Val_CountUp
        if not count_up:
            self.count_direction = DAQmx_Val_CountDown

        if not isinstance(descriptor, bytes):
            descriptor = descriptor.encode()  # type: ignore

        self.task.CreateCICountEdgesChan(
            descriptor,
            None,
            edge=self.edge,
            initialCount=initial_count,
            countDirection=self.count_direction,
        )
        self.task.start()

    def get(self) -> int:
        """
        Return the current edge count.
        """
        self.task.ReadCounterScalarU32(float64(1.0), byref(self.data), None)  # noqa: F405

        self.count = self.data.value

        result = self.count - self.offset
        return result

    def set(self, count: int) -> None:
        """
        Set the count to the desired value.

        Args:
            count (int): the new counter value
        """
        self.offset = self.count + count

    def reset(self) -> None:
        """
        Reset the counter to 0.
        """
        self.set(count=0)

    def close(self) -> None:
        """
        Close the abstractor by clearing the task.
        """
        try:
            self.task.ClearTask()
        except DAQError:
            pass

    def __del__(self) -> None:
        self.close()

    def __str__(self) -> str:
        return str(self.get())


if __name__ == "__main__":
    import time

    ctr0 = EdgeCounterInput(descriptor="Dev1/ctr0", initial_count=0, rising_edge=True, count_up=True)

    for ii in range(100):
        print(ctr0.get())
        # exit(0)
        time.sleep(0.1)

    print("reset..")
    ctr0.reset()

    for ii in range(100):
        print(ctr0.get())
        # exit(0)
        time.sleep(0.1)
